@@ -35,6 +35,14 @@ import os
 import json
 import time
 import re
+import sys
+import io
+
+try:
+    import optimism
+    OPTIMISTIC = True
+except Exception:
+    OPTIMISTIC = False
 
 from . import html_tools
 from . import phrasing
@@ -65,7 +73,7 @@ def show_all_context_values(context):
     """
     The default display_product function for a Context, this creates a
     separate details node for each context key and puts a string
-    representation of the value for that key in an inner <pre> tag.
+    representation of the value for that key in an inner pre tag.
 
     Generally when a context builder creates only a single key, you
     should use the output of build_context_value_displayer with that
@@ -108,50 +116,50 @@ class Context:
     The base context (and therefore all derived contexts) will always
     include the following `potluck.context_utils.BASE_CONTEXT_SLOTS`:
 
-        - "task_info": A task info dictionary for the current task, which
-            will have a "specification" key containing the specification
-            module for this task.
-        - "username": The user who submitted the code we're evaluating.
-        - "submission_root": The root directory within which the
-            submission we're evaluating can be found.
-        - "default_file": The official name of the default file to evaluate.
-        - "actual_file": The actual name of the default file as submitted.
+    - "task_info": A task info dictionary for the current task, which
+        will have a "specification" key containing the specification
+        module for this task.
+    - "username": The user who submitted the code we're evaluating.
+    - "submission_root": The root directory within which the
+        submission we're evaluating can be found.
+    - "default_file": The official name of the default file to evaluate.
+    - "actual_file": The actual name of the default file as submitted.
 
     The typical slots of derived context dictionaries include:
 
-        - "filename": The file name (without path) of the file we're
-            evaluating.
-        - "file_path": The full path to the file we're evaluating.
-        - "source": A string containing the source code of the submitted
-            module.
-        - "parse_errors": A list of Exception objects which were
-            generated but ignored when parsing the submitted code.
-        - "defs": A mapping from function names to AST nodes covering
-            every `def` node in the submitted code (but note that
-            functions with identical names will shadow each other in this
-            map).
-        - "scope": an AST node to check within (e.g., for a function
-            definition check). See ImplementationCheck for an example of
-            how these contexts are created and used.
-        - "docstrings": A mapping from function names to documentation
-            strings for those functions.
-        - "top_scope": The top-level AST node for the submitted code.
-        - "module": a loaded (submitted or solution) module
-            (for a suite of tests that relies on the code).
-        - "output": a string containing output from a function call.
-            This may be the full output, or may be filtered by additional
-            Context objects to contain just part of the full output.
-        - "value": arbitrary Python value returned from a function call.
-        - "trace": a trace list recording certain function calls made
-            during a test, along with arbitrary state snapshots (see
-            `potluck.harness.tracing_function_calls`).
-        - "image_info": an image info dictionary that include a filename
-            which points to an image generated by a function call.
-        - Other keys not defined here may be established and/or used by
-            certain Context or Goal classes.
-        - Versions of the keys above with "ref_" as a prefix hold
-            equivalent values derived from solution instead of submitted
-            code.
+    - "filename": The file name (without path) of the file we're
+        evaluating.
+    - "file_path": The full path to the file we're evaluating.
+    - "source": A string containing the source code of the submitted
+        module.
+    - "parse_errors": A list of Exception objects which were
+        generated but ignored when parsing the submitted code.
+    - "defs": A mapping from function names to AST nodes covering
+        every `def` node in the submitted code (but note that
+        functions with identical names will shadow each other in this
+        map).
+    - "scope": an AST node to check within (e.g., for a function
+        definition check). See ImplementationCheck for an example of
+        how these contexts are created and used.
+    - "docstrings": A mapping from function names to documentation
+        strings for those functions.
+    - "top_scope": The top-level AST node for the submitted code.
+    - "module": a loaded (submitted or solution) module
+        (for a suite of tests that relies on the code).
+    - "output": a string containing output from a function call.
+        This may be the full output, or may be filtered by additional
+        Context objects to contain just part of the full output.
+    - "value": arbitrary Python value returned from a function call.
+    - "trace": a trace list recording certain function calls made
+        during a test, along with arbitrary state snapshots (see
+        `potluck.harness.tracing_function_calls`).
+    - "image_info": an image info dictionary that include a filename
+        which points to an image generated by a function call.
+    - Other keys not defined here may be established and/or used by
+        certain Context or Goal classes.
+    - Versions of the keys above with "ref_" as a prefix hold
+        equivalent values derived from solution instead of submitted
+        code.
 
     Note: For various reasons, the context building process creates
     shallow copies of contexts, not deep copies (the context values, such
@@ -231,6 +239,22 @@ class Context:
         self.cached_value = None
         self.cache_timestamp = None
 
+        self.working_from = None
+        # which base context we're currently working from, for better
+        # exception reporting
+
+    def __str__(self):
+        """
+        Uses the first description entry to hint which context this is.
+        """
+        return 'Context "' + self.feedback_topic() + '"'
+
+    def __repr__(self):
+        """
+        Weaves in the description.
+        """
+        return f'<{type(self).__name__} "{self.feedback_topic()}">'
+
     def __copy__(self):
         """
         Contexts may not be copied. They are entangled with each other,
@@ -279,6 +303,22 @@ class Context:
         A base context is required, and should be a dictionary; normally
         it should have all the `potluck.context_utils.BASE_CONTEXT_SLOTS`
         already populated.
+
+        The resulting context will have a special "__builder__" slot
+        which contains a reference to this `Context` object. Of course,
+        this slot will get overwritten by each context in a chain, so it
+        only stores the last builder to touch the context (but see below).
+
+        The resulting context will have a special "__unmerged__" slot
+        which contains a list of context dictionaries for each dependency
+        of this context, holding the context state created by just that
+        dependency before merging with later dependencies. This can be
+        used to retrieve separate values for the same slot from multiple
+        dependencies, where only the last of those values would normally
+        be retained. Those context dictionaries will of course have
+        "__builder__" and "__unmerged__" slots, so the full chain of
+        `Context` objects responsible for a given context can be
+        recursively retrieved if necessary.
         """
         rerun = False
         if not self.cache_values:
@@ -302,24 +342,35 @@ class Context:
             return result
         else:
             # Create new cached value and update our timestamp
+            self.working_from = base_context
             prev_context = copy.copy(base_context)
             prev_context.update(self.base)
+            unmerged = []
             for dep in self.depends:
                 try:
-                    prev_context.update(dep.create(base_context))
+                    dep_context = dep.create(base_context)
+                    prev_context.update(dep_context)
+                    unmerged.append(dep_context)
                 except Exception as e:
-                    e.html_tb = html_tools.html_traceback()
+                    e.html_tb = html_tools.html_traceback(
+                        linkable=context_utils.linkmap(prev_context)
+                    )
                     raise context_utils.ContextCreationError(
                         self,
                         "Dependency failed.",
                         e
                     )
+            prev_context["__unmerged__"] = unmerged
+            prev_context["__builder__"] = self
+
             try:
                 our_results = self.builder(prev_context)
                 prev_context.update(our_results)
                 self.cached_value = prev_context
             except Exception as e:
-                e.html_tb = html_tools.html_traceback()
+                e.html_tb = html_tools.html_traceback(
+                    linkable=context_utils.linkmap(prev_context)
+                )
                 if isinstance(self.failure_explanation, str):
                     msg = self.failure_explanation
                 elif self.failure_explanation:
@@ -458,19 +509,57 @@ class Context:
 """
 
 
+def add_context_numbering(all_context_objs):
+    """
+    Takes a list of Context objects and looks for objects with duplicated
+    short descriptions, adding numerical suffixes to these.
+    """
+    # Map topics to lists of contexts
+    by_topic = {}
+    by_feedback_topic = {}
+    for ctx in all_context_objs:
+        topic = ctx.description[0]
+        by_topic.setdefault(topic, []).append(ctx)
+
+        if len(ctx.description) > 2:
+            by_feedback_topic.setdefault(ctx.description[2], []).append(ctx)
+
+    # Assign numbers to topics that are duplicated
+    for topic in by_topic:
+        contexts = by_topic[topic]
+        if len(contexts) > 1:
+            # these duplicates need numbering
+            for i, ctx in enumerate(contexts):
+                ctx.description = (
+                    ctx.description[0] + f" #{i+1}",
+                ) + ctx.description[1:]
+
+    # Repeat for feedback topics (numbering will hopefully be consistent
+    # because of consistent iteration order over all context objects, but
+    # if it's not, too bad.
+    for topic in by_feedback_topic:
+        contexts = by_feedback_topic[topic]
+        if len(contexts) > 1:
+            # these duplicates need numbering
+            for i, ctx in enumerate(contexts):
+                ctx.description = ctx.description[:2] + (
+                    ctx.description[2] + f" #{i+1}",
+                ) + ctx.description[3:]
+
+
 def build_context_graph(all_context_objs):
     """
     Takes a list of Context objects and traces dependencies to build a
     top-down rather than bottom-up graph. The graph consists of a list of
     top-level node dictionaries with the following keys:
 
-        context: The Context object for this node.
-        children: A list containing pointers to the node dictionaries of
-            each Context that depends on this one.
-        parents: A list containing pointers to the node dictionaries of
-            each Context that this one depends on.
-        level: An integer indicating the longest chain of ancestors that
-            this node has (0 for nodes without dependencies).
+    - context: The Context object for this node.
+    - children: A list containing pointers to the node dictionaries of
+        each Context that depends on this one.
+    - parents: A list containing pointers to the node dictionaries of
+        each Context that this one depends on.
+    - level: An integer indicating the longest chain of ancestors that
+        this node has (0 for nodes without dependencies).
 
     The exact same node dictionary may appear as a child and/or parent of
     multiple other nodes.
@@ -546,7 +635,7 @@ def assign_cgraph_level(node):
 
 def render_context_graph(cgraph, in_feedback=False):
     """
-    Renders a context graph (see build_context_graph) into HTML.
+    Renders a context graph (see `build_context_graph`) into HTML.
 
     The in_feedback argument controls whether context items should show
     their full or redacted versions.
@@ -568,23 +657,23 @@ def render_context_graph(cgraph, in_feedback=False):
 
 def list_and_render_contexts(cgraph, base_context=None):
     """
-    Transforms a context graph (see build_context_graph) into a list of
+    Transforms a context graph (see `build_context_graph`) into a list of
     context summaries, which are dictionaries with the following keys:
 
-        description: An HTML code string that describes the context item,
-            including a detailed description behind a help button.
-        depends: A list containing integer indices into the entire
-            context list of the other contexts on which this context
-            depends. Will be empty for contexts without dependencies.
-        level: An integer specifying how far this context should be
-            indented, which will be one greater than the maximum level of
-            contexts that this one depends on, starting with 0 for
-            contexts that have no dependencies.
-        value: An HTML code string that displays or summarizes the value
-            produced by this context. This will be the result of the
-            context's display_product function run on the most recent
-            result of that context (or on a fresh result if caching is
-            disabled for the context).
+    - description: An HTML code string that describes the context item,
+        including a detailed description behind a help button.
+    - depends: A list containing integer indices into the entire context
+        list of the other contexts on which this context depends. Will be
+        empty for contexts without dependencies.
+    - level: An integer specifying how far this context should be
+        indented, which will be one greater than the maximum level of
+        contexts that this one depends on, starting with 0 for contexts
+        that have no dependencies.
+    - value: An HTML code string that displays or summarizes the value
+        produced by this context. This will be the result of the
+        context's display_product function run on the most recent result
+        of that context (or on a fresh result if caching is disabled for
+        the context).
 
     The base_context argument is used to generate context results (will
     only happen if cached results are not available or caching is
@@ -1157,12 +1246,21 @@ class ModuleContext(AutoContext):
     """
     Requires a "top_scope" slot (see `CodeContext` which must hold an
     entire module's AST, and creates a "module" slot which holds the
-    module object that results from running that code.
+    module object that results from running that code. Also accesses the
+    "task_info" slot to retrieve the helper_files list from the
+    specification so that it can create a sandbox for running the module
+    code with helper files available.
+
+    In addition to the "module" slot, it creates an "expectations" slot
+    which holds a snapshot of the module's ALL_EXPECTATIONS list, if the
+    optimism module is available.
     """
+    _src = "file_path"
     _from = "top_scope"
     _to = "module"
+    _to_expectations = "expectations"
     _description = (
-        "The values defined by the code.",
+        "The values defined by the code",
         (
             "We will run your code and so that we can run tests on the"
             " values it defines."
@@ -1175,21 +1273,40 @@ class ModuleContext(AutoContext):
         loaded module.
         """
         loaded = context[self._to]
+        tests = {
+            id(exp["case"]): exp["case"]
+            for exp in context[self._to_expectations]
+        }
         defined = [
             name
             for name in dir(loaded)
             if not name.startswith("__") or not name.endswith("__")
         ]
         if len(defined) == 0:
-            return "No values were defined in the file."
+            result = "No values were defined in the file."
         else:
-            return (
+            result = (
                 "The following values were defined in the file:\n"
               + html_tools.build_list(
                     "<code>{}</code>".format(name)
                     for name in defined
                 )
             )
+        if tests:
+            result += (
+                "<br>\nThe module established the following test"
+              + " cases:\n"
+              + html_tools.build_list(
+                  "<code>{}</code>".format(
+                      tests[tid].get("expr_src", "???")
+                  )
+                  for tid in tests
+              )
+            )
+        else:
+            result += "<br>\nThe module didn't create any test cases."
+
+        return result
 
     def __init__(self, depends=None, hidden=False, prep=None, wrap=None):
         """
@@ -1212,9 +1329,12 @@ class ModuleContext(AutoContext):
             in the "top_scope" slot. Actually, uses self._from and
             self._to to determine the slots to read/write, since it can
             also be used to create "ref_module" from "ref_top_scope".
+            Also uses self._src if available to find the source file.
             """
             # Fetch the AST node that we'd like to turn into a module
             node = context_utils.extract(prev_context, self._from)
+            tinfo = context_utils.extract(prev_context, "task_info")
+            spec = tinfo["specification"]
 
             # Prefix the file name so that submitted and solution
             # modules with the same name don't collide
@@ -1227,26 +1347,69 @@ class ModuleContext(AutoContext):
                 prefix = "loaded_"
             full_name = prefix + filename
 
+            # Figure out our file source if we can
+            src_path = prev_context.get(self._src)
+            if src_path:
+                src_path = os.path.abspath(src_path)
+
             # Run the prep function if one was supplied
             if prep is not None:
                 prep(prev_context)
 
+            # Set up phony stdin, so that stray inputs won't immediately
+            # crash the program (if their results are used in a delicate
+            # manner, they still will of course).
+            old_stdin = sys.stdin
+            sys.stdin = context_utils.MashEnter()
+            # Note: we don't care about echoing inputs to stdout here...
+
+            # Set up phony stdout and stderr
+            old_stdout = sys.stdout
+            sys.stdout = io.StringIO()
+            old_stderr = sys.stderr
+            sys.stderr = io.StringIO()
+
+            if OPTIMISTIC:
+                # Ask for normal failure messages
+                optimism.detailLevel(0)
+                # Get rid of any leftover expectations
+                optimism.ALL_EXPECTATIONS = []
+
             # Actually load the module
             try:
-                module = load.create_module_from_code(node, full_name)
+                module = load.create_module_in_sandbox(
+                    node,
+                    full_name,
+                    sandbox_links=load.link_mapping(spec),
+                    on_disk=src_path
+                )
+                if OPTIMISTIC:
+                    established_expectations = optimism.deepish_copy(
+                        optimism.ALL_EXPECTATIONS
+                    )
+                else:
+                    established_expectations = None
             except Exception as e:
                 raise context_utils.ContextCreationError(
                     self,
                     "Unable to run code.",
                     e
                 )
+            finally: # clean up input/output streams
+                sys.stdin = old_stdin
+                sys.stdout = old_stdout
+                sys.stderr = old_stderr
 
             # Wrap our module result if necessary
             if wrap is not None:
                 module = wrap(module)
 
             # Return our new slot
-            return { self._to: module }
+            result = { self._to: module }
+            if OPTIMISTIC:
+                result[self._to_expectations] = established_expectations
+
+            return result
 
         # Figure out if we need to use automatic dependencies:
         if depends is None:
@@ -1268,8 +1431,15 @@ class ModuleContext(AutoContext):
 
 
 # Register a factory for a default ModuleContext as an on-demand option
-# for the "module" slot.
-AutoContext.on_demand((lambda: ModuleContext()), "module")
+# for the "module" and "expectations" slots, or just for "module" if the
+# optimism module isn't available.
+if OPTIMISTIC:
+    AutoContext.on_demand(
+        (lambda: ModuleContext()),
+        "module", "expectations"
+    )
+else:
+    AutoContext.on_demand((lambda: ModuleContext()), "module")
 
 
 class SolnModuleContext(ModuleContext):
@@ -1278,11 +1448,16 @@ class SolnModuleContext(ModuleContext):
     "ref_top_scope" slot (see `SolnCodeContext`) which must hold the
     solution module's AST, and creates a "ref_module" slot which holds
     the module object that results from running that code.
+
+    If the optimism module is available, also creates a
+    "ref_expectations" slot.
     """
+    _src = "ref_file_path"
     _from = "ref_top_scope"
     _to = "ref_module"
+    _to_expectations = "ref_expectations"
     _description = (
-        "The values defined by the solution code.",
+        "The values defined by the solution code",
         (
             "We will run the solution code and so that we can compare"
             " its results to the results of your code."
@@ -1295,8 +1470,15 @@ class SolnModuleContext(ModuleContext):
 
 
 # Register a factory for a default SolnModuleContext as an on-demand
-# option for the "ref_module" slot.
-AutoContext.on_demand((lambda: SolnModuleContext()), "ref_module")
+# option for the "ref_module" and "ref_expectations" slots, or just
+# "ref_module" if the optimism module isn't available.
+if OPTIMISTIC:
+    AutoContext.on_demand(
+        (lambda: SolnModuleContext()),
+        "ref_module", "ref_expectations"
+    )
+else:
+    AutoContext.on_demand((lambda: SolnModuleContext()), "ref_module")
 
 
 class DefinitionsContext(AutoContext):
@@ -1554,7 +1736,7 @@ def build_context_value_displayer(
                     try:
                         ref_rep = json.dumps(context["ref_" + key], indent=2)
                     except TypeError:
-                        ref_rep = repr(context["ref_" + key], indent=2)
+                        ref_rep = repr(context["ref_" + key])
                 ref_rep_html = f"<pre class='context-value'>{ref_rep}</pre>"
             else:
                 ref_rep = ""
@@ -1597,8 +1779,8 @@ def build_simple_context_value_displayer(
 ):
     """
     Creates a display_product function similar to the
-    build_context_value_displayer result, but for simple values which
-    don't need a <pre> wrapper and which can be displayed side-by-side
+    `build_context_value_displayer` result, but for simple values which
+    don't need a pre wrapper and which can be displayed side-by-side
     (e.g., numbers). No diff is included, as it's presumed that any
     differences will be obvious, and values are converted to strings
     using str() instead of json.dumps.
@@ -1836,7 +2018,7 @@ class SiftedContext(Context):
                 if not slot.startswith("ref_")
             )
             description = (
-                f"Certain parts of the {stuff}.",
+                f"Certain parts of the {stuff}",
                 (
                     f"We will search for the pattern"
                     f"<pre><code>{pattern.pattern}</code></pre> within"
