@@ -175,13 +175,13 @@ def test_specification(task_info, examples_dir):
             expectations = by_user[username]
             logging.log(f"Checking expectations for '{username}'...")
             # Evaluate our rubric against the example submission
-            report = spec.rubric.evaluate(
+            evaluation = spec.rubric.evaluate(
                 task_info,
                 username,
                 submission_target
             )
             # Check the resulting report
-            passed, expl = meta.check_entire_report(report, expectations)
+            passed, expl = meta.check_entire_report(evaluation, expectations)
             # Log the resulting explanation
             logging.log(expl)
             status = "passed"
@@ -189,6 +189,20 @@ def test_specification(task_info, examples_dir):
                 status = "FAILED"
                 failed.append(username)
                 any_failed = True
+                # Render failed report for inspection
+                if os.path.isdir("reports"):
+                    cfdir = os.path.join("reports", "__checks__")
+                    os.makedirs(cfdir, exist_ok=True)
+                    fnbase = os.path.join(
+                        cfdir,
+                        f"{task_info['id']}-{username}"
+                    )
+                    report.render_report(
+                        evaluation,
+                        fnbase + ".json",
+                        fnbase + ".html",
+                    )
+                    logging.log(f"Wrote report to '{fnbase}.html'")
             logging.log(f"...done checking '{username}' ({status}).")
         if len(failed) > 0:
             logging.log(f"{len(failed)}/{len(by_user)} examples failed.")
@@ -201,17 +215,31 @@ def test_specification(task_info, examples_dir):
         "soln",
         task_info["target"]
     )
-    soln_report = spec.rubric.evaluate(
+    soln_evaluation = spec.rubric.evaluate(
         task_info,
         "__soln__",
         soln_file
     )
     # Check just defaults for solution report
-    passed, expl = meta.check_entire_report(soln_report, [])
+    passed, expl = meta.check_entire_report(soln_evaluation, [])
     status = "passed"
     if not passed:
         status = "FAILED"
         any_failed = True
+        # Render failed report for inspection
+        if os.path.isdir("reports"):
+            cfdir = os.path.join("reports", "__checks__")
+            os.makedirs(cfdir, exist_ok=True)
+            fnbase = os.path.join(
+                cfdir,
+                f"{task_info['id']}-__soln__"
+            )
+            report.render_report(
+                soln_evaluation,
+                fnbase + ".json",
+                fnbase + ".html",
+            )
+            logging.log(f"Wrote report to '{fnbase}.html'")
     logging.log(expl)
     logging.log(f"Check of solution code {status}.")
 
@@ -401,7 +429,7 @@ def launch_specifications_test(
     log_file=None
 ):
     """
-    Loads a specification and checks any `potluck.meta.Expecation`s
+    Loads a specification and checks any `potluck.meta.Expectation`s
     defined there. Note that corresponding test submissions must already
     be present in the evaluation directory.
 
