@@ -10,6 +10,7 @@ import textwrap
 from . import mast
 from . import harness
 from . import html_tools
+from . import phrasing
 
 
 def summarize_parse_error(e):
@@ -33,6 +34,21 @@ def summarize_parse_error(e):
     # TODO: Any file locations to obfuscate in str(e)?
 
 
+def direct_args_repr(*posargs, **kwargs):
+    """
+    Returns a string representing the arguments provided, such that if
+    put in parentheses after a function name, the resulting string could
+    be evaluated to reproduce a function call (modulo inexact reprs).
+
+    Note that with big argument values, this string will be prohibitively
+    long.
+    """
+    return (
+        ', '.join(repr(arg) for arg in posargs)
+      + ', '.join(f"{key}={kwargs[key]!r}" for key in kwargs)
+    )
+
+
 #------------------------#
 # Docstring manipulation #
 #------------------------#
@@ -163,8 +179,7 @@ def code_check_description(
     looks for a for loop and has a sub-rule that looks for an
     accumulation pattern, the arguments could be:
 
-        a for loop,
-        a for loop with an accumulator,
+    `"a for loop", "a for loop with an accumulator"`
 
     The details_prefix will be prepended to the details part of the
     description that is created.
@@ -244,19 +259,48 @@ def payload_description(
     base_constructor,
     constructor_args,
     augmentations,
-    obfuscated=False
+    obfuscated=False,
+    topic_repr_limit=80,
+    details_repr_limit=80
 ):
     """
-    Returns an HTML description of the test payload that would be
-    constructed using the provided payload constructor, arguments to that
-    constructor (as a dictionary) and augmentations dictionary (mapping
-    augmentation function names to argument dictionaries).
+    Returns an pair of HTML topic and details strings for the test
+    payload that would be constructed using the provided payload
+    constructor, arguments to that constructor (as a dictionary) and
+    augmentations dictionary (mapping augmentation function names to
+    argument dictionaries).
 
     If `obfuscated` is set to True, an obfuscated description will
     be returned which avoids key details like which specific
     arguments are used or what inputs are provided. This also puts
     the description into the future tense instead of the past tense.
+
+    `topic_repr_limit` and  `details_repr_limit` controls the character
+    counts at which direct representations of arguments are considered
+    too long for the topic (alternative is to omit them) or for the
+    details (alternative is to use a bulleted list of smart reprs).
     """
+    # Prefix that describes what value(s) are created
+    products = [ "result" ]
+    if "capturing_printed_output" in augmentations:
+        if base_constructor == harness.create_module_import_payload:
+            products = [ "output" ]
+        else:
+            products = [ "result", "output" ]
+
+    if "tracing_function_calls" in augmentations:
+        products.append("process trace")
+
+    what = phrasing.comma_list(products)
+
+    # Tense management
+    was = "was"
+    were = "were"
+    if obfuscated:
+        was = "will be"
+        were = "will be"
+
+    # Base topic/details based on constructor type
     if base_constructor == harness.create_run_function_payload:
         fname = constructor_args["fname"]
         posargs = constructor_args["posargs"]
@@ -264,27 +308,33 @@ def payload_description(
         # Note: copy_args does not show up in descriptions
 
         if not posargs and not kwargs: # if there are no arguments
-            args_repr = ""
+            topic_repr = f"<code>{fname}</code>"
+            details_repr = topic_repr
         elif obfuscated:
-            args_repr = " with some arguments"
+            topic_repr = f"<code>{fname}(...)</code>"
+            details_repr = f"<code>{fname}</code> with some arguments"
         else:
-            args_repr = (
-                " with the following arguments:\n"
-              + html_tools.args_repr_list(posargs, kwargs)
-            )
+            direct = html_tools.escape(direct_args_repr(*posargs, **kwargs))
+            if len(fname + direct) > topic_repr_limit:
+                topic_repr = f"<code>{fname}(...)</code>"
+            else:
+                topic_repr = f"<code>{fname}({direct})</code>"
 
+            if len(direct) > details_repr_limit:
+                details_repr = (
+                    f"<code>{fname}</code> with the following"
+                    f" arguments:\n"
+                ) + html_tools.args_repr_list(posargs, kwargs)
+            else:
+                details_repr = f"<code>{fname}({direct})</code>"
+
+        topic = f"The {what} of {topic_repr}"
         if obfuscated:
-            result = (
-                f"We will run your <code>{fname}</code>"
-                f" function{args_repr}"
-            )
+            details = f"We will run {details_repr} and record the {what}."
         else:
-            result = (
-                f"We ran your <code>{fname}</code>"
-                f" function{args_repr}"
-            )
+            details = f"We ran {details_repr} and recorded the {what}."
 
-    if base_constructor == harness.create_run_harness_payload:
+    elif base_constructor == harness.create_run_harness_payload:
         test_harness = constructor_args["harness"]
         fname = constructor_args["fname"]
         posargs = constructor_args["posargs"]
@@ -292,23 +342,45 @@ def payload_description(
         # Note: copy_args does not show up in descriptions
 
         if not posargs and not kwargs: # if there are no arguments
-            args_repr = ""
+            topic_args_repr = ""
+            details_args_repr = ""
         elif obfuscated:
-            args_repr = " with some arguments"
+            topic_args_repr = "(...)"
+            details_args_repr = " with some arguments"
         else:
-            args_repr = (
-                " with the following arguments:\n"
-              + html_tools.args_repr_list(posargs, kwargs)
-            )
+            direct = html_tools.escape(direct_args_repr(*posargs, **kwargs))
+            if len(fname + direct) > topic_repr_limit:
+                topic_args_repr = "(...)"
+            else:
+                topic_args_repr = "(" + direct + ")"
 
-        _, obfuscated, _, clear = harness_descriptions(test_harness, fname)
+            if len(direct) > details_repr_limit:
+                details_args_repr = (
+                    " with the following arguments:\n"
+                  + html_tools.args_repr_list(posargs, kwargs)
+                )
+            else:
+                details_args_repr = (
+                    f" with arguments: <code>({direct})</code>"
+                )
+
+        (
+            obf_topic, obfuscated,
+            clear_topic, clear
+        ) = harness_descriptions(
+            test_harness,
+            fname,
+            topic_args_repr,
+            details_args_repr,
+            what
+        )
 
         if obfuscated:
-            result = obfuscated
+            topic = obf_topic
+            details = obfuscated
         else:
-            result = clear
-
-        result += args_repr
+            topic = clear_topic
+            details = clear
 
     elif base_constructor == harness.create_module_import_payload:
         prep = constructor_args["prep"]
@@ -319,46 +391,61 @@ def payload_description(
             mod = ""
 
         if obfuscated:
-            result = f"We will run your submitted code{mod}"
+            topic = "The {what} of your program"
+            details = (
+                f"We will run your submitted code{mod} and record"
+                f" the {what}."
+            )
         else:
-            result = f"We ran your submitted code{mod}"
+            topic = "The {what} of your program"
+            details = (
+                f"We ran your submitted code{mod} and recorded"
+                f" the {what}."
+            )
 
     elif base_constructor == harness.create_read_variable_payload:
         varname = constructor_args["varname"]
         if obfuscated:
-            result = (
+            topic = "The value of <code>{varname}</code>"
+            details = (
                 f"We will inspect the value of"
-                f" <code>{varname}</code>"
+                f" <code>{varname}</code>."
             )
         else:
-            result = (
-                f"We inspected the value of <code>{varname}</code>"
+            topic = "The value of <code>{varname}</code>"
+            details = (
+                f"We inspected the value of <code>{varname}</code>."
             )
 
     else: # unsure what our payload is (this shouldn't happen)...
         if obfuscated:
-            result = "We will test your submission."
+            topic = "A test of your code"
+            details = "We will test your submission."
         else:
-            result = "We tested your submission."
+            topic = "A test of your code"
+            details = "We tested your submission."
 
     # Assemble details from augmentations
-    details = []
+    testing_details = []
     if "with_timeout" in augmentations:
         limit = augmentations["with_timeout"]["time_limit"]
-        details.append(
-            f"Will be terminated if it takes longer than {limit}s."
-        )
+        if obfuscated:
+            testing_details.append(
+                f"Will be terminated if it takes longer than {limit}s."
+            )
+        else:
+            testing_details.append(f"Ran with a {limit}s time limit.")
 
     if "capturing_printed_output" in augmentations:
         errors_too = augmentations["capturing_printed_output"]\
             .get("capture_errors")
 
         if errors_too:
-            details.append(
-                "Printed output and error messages will be recorded."
+            testing_details.append(
+                f"Printed output and error messages {were} recorded."
             )
         else:
-            details.append("Printed output will be recorded.")
+            testing_details.append(f"Printed output {was} recorded.")
 
     if "with_fake_input" in augmentations:
         inputs = augmentations["with_fake_input"]["inputs"]
@@ -367,10 +454,25 @@ def payload_description(
         policy_note = " in a loop" if policy == "loop" else ""
 
         if obfuscated:
-            details.append("Inputs will be provided")
+            topic += " with inputs"
+            testing_details.append("Inputs will be provided")
         else:
-            details.append(
-                f"The following inputs will be provided{policy_note}:\n"
+            listing = ', '.join(
+                f"<code>{html_tools.escape(repr(inp))}</code>"
+                for inp in inputs
+            )
+            inputs_pl = phrasing.plural(len(inputs), "input")
+            was_were = phrasing.plural(len(inputs), "was", "were")
+            proposed = f"{topic} with {inputs_pl}: {listing}"
+            if html_tools.len_as_text(topic + listing) < topic_repr_limit:
+                topic = proposed
+            else:
+                topic += f" with {inputs_pl}"
+            testing_details.append(
+                (
+                    f"The following {inputs_pl} {was_were}"
+                    f" provided{policy_note}:\n"
+                )
               + html_tools.build_list(
                     html_tools.dynamic_html_repr(text)
                     for text in inputs
@@ -380,8 +482,8 @@ def payload_description(
     if "with_module_decorations" in augmentations:
         args = augmentations["with_module_decorations"]
         decmap = args["decorations"]
-        details.append(
-            "Adjustments will be made to the following functions:\n"
+        testing_details.append(
+            "Adjustments {were} made to the following functions:\n"
           + html_tools.build_list(
                 "<pre>{fn}</pre>"
                 for fn in decmap
@@ -399,9 +501,9 @@ def payload_description(
         else:
             tracking = f" ({sfdesc[0]})"
 
-        details.append(
+        testing_details.append(
             (
-                f"Calls to the following functions will be"
+                f"Calls to the following function(s) {were}"
                 f" monitored{tracking}:\n"
             )
           + html_tools.build_list(
@@ -413,8 +515,8 @@ def payload_description(
     if "sampling_distribution_of_results" in augmentations:
         args = augmentations["sampling_distribution_of_results"]
         trials = args["trials"]
-        details.append(
-            f"The distribution of results will be measured across"
+        testing_details.append(
+            f"The distribution of results {was} measured across"
             f" {trials} trials."
         )
 
@@ -422,15 +524,21 @@ def payload_description(
     # run_for_base_and_ref_values, as comparing to the solution
     # value is implied.
 
-    result += (
+    details += (
         "<br>\nTesting details:"
-      + html_tools.build_list(details)
+      + html_tools.build_list(testing_details)
     )
 
-    return result
+    return (topic, details)
 
 
-def harness_descriptions(test_harness, fname):
+def harness_descriptions(
+    test_harness,
+    fname,
+    topic_args_repr,
+    details_args_repr,
+    what_is_captured
+):
     """
     Extracts descriptions of a test harness from its docstring and
     formats them using the given function name, returning a description
@@ -438,23 +546,32 @@ def harness_descriptions(test_harness, fname):
     """
     hdesc = description_templates_from_docstring(test_harness)
 
-    if hdesc[0] == test_harness.__name__: # no custom description
-        result = (
-            f"Specialized test of <code>{fname}</code>",
+    # Create default description if there is no custom description
+    if hdesc[0] == test_harness.__name__:
+        hdesc = (
+            "Specialized test of <code>{fname}{args}</code>",
             (
-                f"We will test your <code>{fname}</code> using"
-                f" <code>{hdesc[0]}</code>"
+                "We will test your <code>{fname}</code>{args} using"
+              + " <code>" + hdesc[0] + "</code>, recording the"
+              + " {captured}."
             ),
-            f"Specialized test of <code>{fname}</code>",
+            "Specialized test of <code>{fname}{args}</code>",
             (
-                "We tested your <code>{fname}</code> using"
-                f" <code>{hdesc[0]}</code>"
+                "We tested your <code>{fname}</code>{args} using"
+              + " <code>" + hdesc[0] + "</code>, recording the"
+              + " {captured}."
             )
         )
-    else:
-        result = (
-            part.format(fname=fname)
-            for part in hdesc
-        )
+
+    args_for_parts = [
+        topic_args_repr,
+        details_args_repr,
+        topic_args_repr,
+        details_args_repr,
+    ]
+    result = (
+        part.format(fname=fname, args=args_repr, captured=what_is_captured)
+        for part, args_repr in zip(hdesc, args_for_parts)
+    )
 
     return result
