@@ -16,6 +16,13 @@ Format for timestamps (see `time.strftime`).
 """
 
 
+INITIAL_CWD = os.getcwd()
+"""
+The working directory on launch (e.g., not a temporary sandbox
+directory).
+"""
+
+
 def timestamp(when=None):
     """
     Returns a timestamp string based on the current time, or based on a
@@ -71,7 +78,7 @@ def potluck_src_dir():
     located.
     """
     return os.path.abspath(
-        os.path.join(os.getcwd(), os.path.dirname(__file__))
+        os.path.join(INITIAL_CWD, os.path.dirname(__file__))
     )
 
 
@@ -92,11 +99,10 @@ def get_spec_module_name():
 
     if cf:
         result = cf.f_globals.get("__name__", "unknown")
+        del cf
     else:
         result = "unknown"
 
-    del cf
-
     return result
 
 
@@ -105,7 +111,7 @@ def get_spec_file_name():
     Uses the inspect module to get the path of the specifications file,
     assuming that we're in a function which was ultimately called from
     that module, and that module is the only one in our current call
-    stack whose filename ends with '[/]spec.py'. Returns 'unknown' if it
+    stack whose filename ends with '/spec.py'. Returns 'unknown' if it
     can't find an appropriate call frame in the current stack.
     """
     cf = inspect.currentframe()
