@@ -24,6 +24,8 @@ can't be usefully applied to them).
 import copy
 import sys
 import io
+import re
+import traceback
 
 import turtle
 
@@ -32,6 +34,7 @@ from . import mast
 from . import context_utils
 from . import html_tools
 from . import timeout
+from . import logging
 
 
 #---------#
@@ -83,6 +86,9 @@ def create_module_import_payload(
     module in the "module" slot along with "original_source" and "source"
     slots holding the original and (possibly modified) source code.
 
+    It reads the "task_info" context slot to access the specification and
+    load the helper files list to make available during module execution.
+
     A custom `name_prefix` may be given which will alter the name of the
     imported module in sys.modules and in the __name__ automatic
     variable as the module is being created; use this to avoid conflicts
@@ -111,6 +117,8 @@ def create_module_import_payload(
         filename = context_utils.extract(context, "filename")
         file_path = context_utils.extract(context, "file_path")
         full_name = name_prefix + filename
+        tinfo = context_utils.extract(context, "task_info")
+        spec = tinfo["specification"]
 
         # Read the file
         with open(file_path, 'r', encoding="utf-8") as fin:
@@ -126,24 +134,26 @@ def create_module_import_payload(
         if use_fix_parse:
             # Parse using fix_parse
             fixed, node, errors = load.fix_parse(source, full_name)
-            module = load.create_module_from_code(node, full_name)
-            result = {
-                "original_source": original_source,
-                "source": fixed,
-                "scope": node,
-                "module": module,
-                "parse_errors": errors
-            }
         else:
             # Just parse normally without attempting to steamroll errors
+            fixed = source
             node = mast.parse(source, filename=full_name)
-            module = load.create_module_from_code(node, full_name)
-            result = {
-                "original_source": original_source,
-                "source": source,
-                "scope": node,
-                "module": module
-            }
+            errors = None
+
+        module = load.create_module_in_sandbox(
+            node,
+            full_name,
+            sandbox_links=load.link_mapping(spec),
+            on_disk=file_path
+        )
+        result = {
+            "original_source": original_source,
+            "source": fixed,
+            "scope": node,
+            "module": module
+        }
+        if errors:
+            result["parse_errors"] = errors
 
         # Wrap the resulting module if a wrap function was provided
         if wrap:
@@ -171,7 +181,15 @@ def create_read_variable_payload(module, varname):
         nonlocal varname
         module = context_utils.extract(context, "module")
         if isinstance(varname, context_utils.ContextualValue):
-            varname = varname.replace(context)
+            try:
+                varname = varname.replace(context)
+            except Exception:
+                logging.log(
+                    "Encountered error while attempting to substitute"
+                    " contextual value:"
+                )
+                logging.log(traceback.format_exc())
+                raise
         return { "value": getattr(module, varname) }
 
     return payload
@@ -187,7 +205,9 @@ def create_run_function_payload(
     Creates a payload function which retrieves a function from the
     "module" slot of the given context and runs it with certain
     positional and/or keyword arguments, returning a "value" context slot
-    containing the function's result.
+    containing the function's result. The arguments used are also placed
+    into "args" and "kwargs" context slots in case those are useful for
+    later checks.
 
     If `copy_args` is set to True, deep copies of argument values will be
     made before they are passed to the target function (note that keyword
@@ -211,14 +231,30 @@ def create_run_function_payload(
         nonlocal fname
         module = context_utils.extract(context, "module")
         if isinstance(fname, context_utils.ContextualValue):
-            fname = fname.replace(context)
+            try:
+                fname = fname.replace(context)
+            except Exception:
+                logging.log(
+                    "Encountered error while attempting to substitute"
+                    " contextual value:"
+                )
+                logging.log(traceback.format_exc())
+                raise
         fn = getattr(module, fname)
 
         real_posargs = []
         real_kwargs = {}
         for arg in posargs:
             if isinstance(arg, context_utils.ContextualValue):
-                arg = arg.replace(context)
+                try:
+                    arg = arg.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             if copy_args:
                 arg = copy.deepcopy(arg)
@@ -227,18 +263,38 @@ def create_run_function_payload(
 
         for key in kwargs:
             if isinstance(key, context_utils.ContextualValue):
-                key = key.replace(context)
+                try:
+                    key = key.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             value = kwargs[key]
             if isinstance(value, context_utils.ContextualValue):
-                value = value.replace(context)
+                try:
+                    value = value.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             if copy_args:
                 value = copy.deepcopy(value)
 
             real_kwargs[key] = value
 
-        return { "value": fn(*real_posargs, **real_kwargs) }
+        return {
+            "value": fn(*real_posargs, **real_kwargs),
+            "args": real_posargs,
+            "kwargs": real_kwargs
+        }
 
     return payload
 
@@ -280,14 +336,30 @@ def create_run_harness_payload(
         nonlocal fname
         module = context_utils.extract(context, "module")
         if isinstance(fname, context_utils.ContextualValue):
-            fname = fname.replace(context)
+            try:
+                fname = fname.replace(context)
+            except Exception:
+                logging.log(
+                    "Encountered error while attempting to substitute"
+                    " contextual value:"
+                )
+                logging.log(traceback.format_exc())
+                raise
         fn = getattr(module, fname)
 
         real_posargs = []
         real_kwargs = {}
         for arg in posargs:
             if isinstance(arg, context_utils.ContextualValue):
-                arg = arg.replace(context)
+                try:
+                    arg = arg.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             if copy_args:
                 arg = copy.deepcopy(arg)
@@ -296,11 +368,27 @@ def create_run_harness_payload(
 
         for key in kwargs:
             if isinstance(key, context_utils.ContextualValue):
-                key = key.replace(context)
+                try:
+                    key = key.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             value = kwargs[key]
             if isinstance(value, context_utils.ContextualValue):
-                value = value.replace(context)
+                try:
+                    value = value.replace(context)
+                except Exception:
+                    logging.log(
+                        "Encountered error while attempting to substitute"
+                        " contextual value:"
+                    )
+                    logging.log(traceback.format_exc())
+                    raise
 
             if copy_args:
                 value = copy.deepcopy(value)
@@ -353,6 +441,8 @@ def run_for_base_and_ref_values(payload, used_by_both=None):
                 ref_context[key] = context[key]
             elif key.startswith("ref_"):
                 ref_context[key[4:]] = context[key]
+                # Retain original ref_ slots alongside collapsed slots
+                ref_context[key] = context[key]
 
         # Get results from collapsed context
         try:
@@ -462,10 +552,16 @@ def with_fake_input(payload, inputs, extra_policy="error"):
     """
     Creates a modified payload function which runs the given payload but
     supplies a pre-determined sequence of strings whenever `input` is
-    called instead of actually prompting for values from stdin.
+    called instead of actually prompting for values from stdin. The
+    prompts and input values that would have shown up are still printed,
+    although a pair of zero-width word-joiner characters is added before
+    and after the fake input value at each prompt in the printed output.
 
     The `inputs` and `extra_policy` arguments are passed to
     `create_mock_input` to create the fake input setup.
+
+    The result will have "inputs" and "input_policy" context slots added
+    that store the specific inputs used, and the extra input policy.
     """
     # Create mock input function and input reset function
     mock_input, reset_input = create_mock_input(inputs, extra_policy)
@@ -488,11 +584,35 @@ def with_fake_input(payload, inputs, extra_policy="error"):
         builtins.input = original_input
         reset_input()
 
+        # Add "inputs" and "input_policy" context slots to the result
+        result["inputs"] = inputs
+        result["input_policy"] = extra_policy
+
         return result
 
     return fake_input_payload
 
 
+FAKE_INPUT_PATTERN = (
+    "\u2060\u2060((?:[^\u2060]|(?:\u2060[^\u2060]))*)\u2060\u2060"
+)
+"""
+A regular expression which can be used to find fake input values in
+printed output from code that uses a mock input. The first group of each
+match will be a fake output value.
+"""
+
+
+def strip_mock_input_values(output):
+    """
+    Given a printed output string produced by code using mocked inputs,
+    returns the same string, with the specific input values stripped out.
+    Actually strips any values found between paired word-joiner (U+2060)
+    characters, as that's what mock input values are wrapped in.
+    """
+    return re.sub(FAKE_INPUT_PATTERN, "", output)
+
+
 def create_mock_input(inputs, extra_policy="error"):
     """
     Creates two functions: a stand-in for `input` that returns strings
@@ -502,11 +622,11 @@ def create_mock_input(inputs, extra_policy="error"):
     The extra_inputs_policy specifies what happens if the inputs list
     runs out:
 
-        "loop" means that it will be repeated again, ad infinitum.
-        "hold" means that the last value will be returned for all
-            subsequent input calls.
-        "error" means an `EOFError` will be raised as if stdin had been
-            closed.
+    - "loop" means that it will be repeated again, ad infinitum.
+    - "hold" means that the last value will be returned for all
+        subsequent input calls.
+    - "error" means an `EOFError` will be raised as if stdin had been
+        closed.
 
     "hold" is the default policy.
     """
@@ -518,19 +638,26 @@ def create_mock_input(inputs, extra_policy="error"):
         Function that retrieves the next input from the inputs list and
         behaves according to the extra_inputs_policy when inputs run out:
 
-            If extra_inputs_policy is "hold," the last input is returned
-            repeatedly.
+        - If extra_inputs_policy is "hold," the last input is returned
+          repeatedly.
 
-            If extra_inputs_policy is "loop," the cycle of inputs repeats
-            indefinitely.
+        - If extra_inputs_policy is "loop," the cycle of inputs repeats
+          indefinitely.
 
-            If extra_inputs_policy is "error," (or any other value) an
-            EOFError is raised when the inputs run out. This also happens
-            if the inputs list is empty to begin with.
+        - If extra_inputs_policy is "error," (or any other value) an
+          EOFError is raised when the inputs run out. This also happens
+          if the inputs list is empty to begin with.
 
         This function prints the prompt and the input that it is about to
         return, so that they appear in printed output just as they would
         have if normal input() had been called.
+
+        To enable identification of the input values, a pair of
+        zero-width "word joiner" character (U+2060) is printed directly
+        before and directly after each input value. These should not
+        normally be visible when the output is inspected by a human, but
+        can be searched for (and may also influence word wrapping in some
+        contexts).
         """
         nonlocal input_index
         print(prompt, end="")
@@ -552,7 +679,7 @@ def create_mock_input(inputs, extra_policy="error"):
             result = inputs[input_index]
             input_index += 1
 
-        print(result)
+        print('\u2060\u2060' + result + '\u2060\u2060')
         return result
 
     def reset_input():
@@ -615,26 +742,24 @@ def tracing_function_calls(payload, trace_targets, state_function):
     Each trace entry in the resulting trace represents one function call
     in the outermost scope and is a dictionary with the following keys:
 
-        fname: The name of the function that was called.
-        args: A dictionary of arguments passed to the function, mapping
-            argument names to their values. For calls to C functions
-            (such as most built-in functions), arguments are not
-            available, and this key will not be present.
-        result: The return value of the function. May be None if the
-            function was terminated due to an exception, but there's no
-            way to distinguish that from an intentional None return.
-            For calls to C functions, this key will not be present.
-        pre_state: A state object resulting from calling the given
-            state_function just before the traced function call starts,
-            with the function name as its only argument. Calls made
-            during the execution of the state function will not be
-            traced.
-        post_state: The same kind of state object, but captured right
-            before the return of the traced function.
-        during: A list of trace entries in the same format representing
-            traced function calls which were initiated and returned
-            before the end of the function call that this trace entry
-            represents.
+    - fname: The name of the function that was called.
+    - args: A dictionary of arguments passed to the function, mapping
+        argument names to their values. For calls to C functions (such as
+        most built-in functions), arguments are not available, and this
+        key will not be present.
+    - result: The return value of the function. May be None if the
+        function was terminated due to an exception, but there's no way
+        to distinguish that from an intentional None return. For calls to
+        C functions, this key will not be present.
+    - pre_state: A state object resulting from calling the given
+        state_function just before the traced function call starts, with
+        the function name as its only argument. Calls made during the
+        execution of the state function will not be traced.
+    - post_state: The same kind of state object, but captured right
+        before the return of the traced function.
+    - during: A list of trace entries in the same format representing
+        traced function calls which were initiated and returned before
+        the end of the function call that this trace entry represents.
 
     Note that to inspect all function calls, the hierarchy must be
     traversed recursively to look at calls in "during" slots.
@@ -930,7 +1055,7 @@ class Missing:
 
 class Generic:
     """
-    Class for creating missing parent objects in set_dot_attr.
+    Class for creating missing parent objects in `set_dot_attr`.
     """
     pass
 
@@ -1055,13 +1180,13 @@ def capture_turtle_state(_):
     This state-capture function logs the following pieces of global
     turtle state:
 
-        position: A 2-tuple of x/y coordinates.
-        heading: A floating point number in degrees.
-        pen_is_down: Boolean indicating pen state.
-        is_filling: Boolean indicating whether we're filling or not.
-        pen_size: Floating-point pen size.
-        pen_color: String indicating current pen color.
-        fill_color: String indicating current fill color.
+    - position: A 2-tuple of x/y coordinates.
+    - heading: A floating point number in degrees.
+    - pen_is_down: Boolean indicating pen state.
+    - is_filling: Boolean indicating whether we're filling or not.
+    - pen_size: Floating-point pen size.
+    - pen_color: String indicating current pen color.
+    - fill_color: String indicating current fill color.
 
     This state-capture function ignores its argument (which is the name
     of the function being called).
