@@ -5,16 +5,31 @@ Tools for building HTML strings.
 html_tools.py
 """
 
-
+import sys
 import traceback
 import difflib
-import html
 import os
+import re
 
 from . import phrasing
 from . import file_utils
 
 
+# Figure out how to escape HTML (2/3 compatibility)
+if sys.version_info[0] < 3:
+    # use cgi.escape instead of html.escape
+    import cgi
+    escape = cgi.escape
+else:
+    # use html.escape
+    import html
+    escape = html.escape
+
+
+#---------#
+# Globals #
+#---------#
+
 STATUS_SYMBOLS = {
     "unknown": " ", # non-breaking space
     "not applicable": "–",
@@ -32,6 +47,21 @@ Limit in terms of characters before we try advanced formatting.
 """
 
 
+#-----------#
+# Functions #
+#-----------#
+
+def len_as_text(html_string):
+    """
+    Returns an approximate length of the given string in characters
+    assuming HTML tags would not be visible and HTML entities would be
+    single characters.
+    """
+    without_tags = re.sub("<[^>]*>", "", html_string)
+    without_entities = re.sub("&[#0-9A-Za-z]+;", ".", without_tags)
+    return len(without_entities)
+
+
 def create_help_button(help_content):
     """
     Returns an HTML string for a button that can be clicked to display
@@ -49,15 +79,16 @@ def create_help_button(help_content):
 
 def build_list(items, ordered=False):
     """
-    Builds an HTML <ul> tag, or an <ol> tag if `ordered` is set to True.
+    Builds an HTML ul tag, or an ol tag if `ordered` is set to True.
     Items must be a list of (possibly HTML) strings.
     """
     tag = "ol" if ordered else "ul"
     return (
-        "<{tag}>"
-      + "\n".join("<li>{}</li>".format(item) for item in items)
-      + "\n</{tag}>"
-    ).format(tag=tag)
+        "<{tag}>\n{items}\n</{tag}>"
+    ).format(
+        tag=tag,
+        items="\n".join("<li>{}</li>".format(item) for item in items)
+    )
 
 
 def build_html_details(title, content, classes=None):
@@ -77,17 +108,19 @@ def build_html_details(title, content, classes=None):
 
 def build_html_tabs(tabs):
     """
-    Builds an HTML structure incorporating JS code which uses several tab
-    elements and a scrollable region to implement a multiple-tabs
-    structure. The given tabs list should contain title, content pairs,
-    where both titles and contents are strings that may contain HTML
-    code.
+    Builds an HTML structure which uses several tab elements and a
+    scrollable region to implement a multiple-tabs structure. The given
+    tabs list should contain title, content pairs, where both titles and
+    contents are strings that may contain HTML code.
+
+    `resources/potluck.js` defines the Javascript code necessary for the
+    tabs to work correctly.
     """
     tab_pieces = [
         (
             """
 <li
- class='tab{selected}' onclick='toggleTab();'
+ class='tab{selected}'
  tabindex="0"
  aria-label="Activate to display {title} after this list."
 >
@@ -163,27 +196,26 @@ def block_id(taskid, filename):
 
 def html_link_to_line(taskid, filename, lineno):
     """
-    Returns an HTML <a> tag (as a string) that links to the specified
+    Returns an HTML anchor tag (as a string) that links to the specified
     line of code in the specified file of the specified task.
+
+    `resources/potluck.js` includes code that will add click handlers to
+    these links so that the line being jumped to gets highlighted.
     """
-    blockid = block_id(taskid, filename)
     lineid = line_id(taskid, filename, lineno)
-    focus_call = "focusCodeLine('{blockid}', '{lineid}'); return true;"\
-        .format(blockid=blockid, lineid=lineid)
     return (
-        '<a class="lineref" href="#{lineid}"'
-        ' onclick="{focus_call}">{lineno}</a>'
+        '<a class="lineref" data-lineid="{lineid}"'
+        ' href="#{lineid}">{lineno}</a>'
     ).format(
         lineid=lineid,
-        focus_call=focus_call,
         lineno=lineno
     )
 
 
 def html_output_details(raw_output, title="Output"):
     """
-    Takes raw program output and turns it into an expandable <details> tag
-    with <pre> formatting.
+    Takes raw program output and turns it into an expandable details tag
+    with pre formatting.
     """
     return (
         '<details>\n'
@@ -208,7 +240,7 @@ def html_diff_table(
     """
     Uses difflib to create and return an HTML string that encodes a table
     comparing two (potentially multi-line) outputs. If joint_title is
-    given, the result is wrapped into a <details> tag with that title and
+    given, the result is wrapped into a details tag with that title and
     class diff_table.
 
     If line_limit is not None (the default is 300) then only that many
@@ -367,17 +399,33 @@ def dynamic_html_repr(thing, reasonable=200, limit=10000):
         return build_display_box(truncate(rep, limit))
 
 
-def html_traceback(title=None):
+def html_traceback(exc=None, title=None, linkable=None):
     """
     In an exception handler, returns an HTML string that includes the
     exception type, message, and traceback. Must be called from an except
-    clause.
+    clause, unless an exception object with a __traceback__ value is
+    provided.
 
-    If title is given and not None, a <details> tag will be returned
+    If title is given and not None, a details tag will be returned
     using that title, which can be expanded to show the traceback,
-    otherwise just a <pre> tag is returned containing the traceback.
+    otherwise just a pre tag is returned containing the traceback.
+
+    If linkable is given, it must be a dictionary mapping filenames to
+    task IDs, and line numbers of those files which appear in the
+    traceback will be turned into links to those lines.
     """
-    result = string_traceback()
+    result = escape(string_traceback(exc))
+    if linkable:
+        for fn in linkable:
+            taskid = linkable[fn]
+
+            def replacer(match):
+                link = html_link_to_line(taskid, fn, int(match.group(1)))
+                return f'{fn}&quot;, line {link},'
+
+            pat = rf'{fn}&quot;, line ([0-9]+),'
+            result = re.sub(pat, replacer, result)
+
     pre = '<pre class="traceback">\n{}\n</pre>'.format(result)
     if title is not None:
         return build_html_details(title, pre, "error")
@@ -385,11 +433,32 @@ def html_traceback(title=None):
         return pre
 
 
-def string_traceback():
+REWRITES = {
+    file_utils.potluck_src_dir(): "<potluck>"
+}
+"""
+The mapping from filenames to replacements to be used for rewriting
+filenames in tracebacks.
+"""
+
+
+def set_tb_rewrite(filename, rewrite_as):
+    """
+    Sets up a rewriting rule that will be applied to string and HTML
+    tracebacks. You must provide the filename that should be rewritten,
+    and the string to replace it with. Set rewrite_as to None to remove a
+    previously-established rewrite rule.
+    """
+    global REWRITES
+    REWRITES[filename] = rewrite_as
+
+
+def string_traceback(exc=None):
     """
     When called in an exception handler, returns a multi-line string
     including what Python would normally print: the exception type,
-    message, and a traceback.
+    message, and a traceback. You can also call it anywhere if you can
+    provide an exception object which has a __traceback__ value.
 
     The traceback gets obfuscated by replacing full file paths that start
     with the potluck directory with just the package directory and
@@ -397,26 +466,27 @@ def string_traceback():
     directory with just the task ID and then the file path from the spec
     directory (usually starter/, soln/, or the submitted file itself).
     """
-    psd = file_utils.potluck_src_dir()
     sfd = os.path.split(file_utils.get_spec_file_name())[0]
+    rewrites = {}
+    rewrites.update(REWRITES)
+    if sfd:
+        rewrites[sfd] = "<task>"
 
-    raw = traceback.format_exc()
+    if exc is None:
+        raw = traceback.format_exc()
+    else:
+        raw = ''.join(traceback.format_tb(exc.__traceback__) + [ str(exc) ])
 
-    # TODO: What about submitted files?
-    return rewrite_traceback_filenames(
-        raw,
-        {
-            psd: "potluck",
-            sfd: "<task>"
-        }
-    )
+    return rewrite_traceback_filenames(raw, rewrites)
 
 
 def rewrite_traceback_filenames(raw_traceback, prefix_map):
     """
     Accepts a traceback as a string, and returns a modified string where
     filenames have been altered by replacing the keys of the given
-    prefix_map with their values.
+    prefix_map with their values. In addition, filenames which include a
+    directory that ends in "__tmp" will have all directory entries up to
+    and including that one stripped from their path.
     """
     result = raw_traceback
     for prefix in prefix_map:
@@ -426,6 +496,12 @@ def rewrite_traceback_filenames(raw_traceback, prefix_map):
             'File "{replace}'.format(replace=replace)
         )
 
+    result = re.sub(
+        'File ".*__tmp' + os.path.sep,
+        'File "<submission>/',
+        result
+    )
+
     return result
 
 
@@ -463,6 +539,15 @@ def function_def_code_tags(fn_name, args_pattern, announce=None):
         junction="or"
     )
 
+    # Add a comment about the number of arguments required
+    if isinstance(args_pattern, int):
+        with_n = " with {} {}".format(
+            args_pattern,
+            phrasing.plural(args_pattern, "argument")
+        )
+        code_tag += with_n
+        details_code += with_n
+
     return code_tag, details_code
 
 
@@ -474,7 +559,7 @@ def function_call_code_tags(fn_name, args_pattern, is_method=False):
     `function_def_code_tags`, except that it works for a function call
     instead of a function definition.
 
-    If the args_pattern is "<any arguments>", the parentheses for the
+    If the args_pattern is "-any arguments-", the parentheses for the
     function call will be omitted entirely.
     """
     if isinstance(fn_name, str):
@@ -484,7 +569,7 @@ def function_call_code_tags(fn_name, args_pattern, is_method=False):
 
     # If there are specific args we can give users more info about what
     # they need to do.
-    if args_pattern == "<any arguments>":
+    if args_pattern == "-any arguments-":
         specific_names = names
     elif isinstance(args_pattern, str):
         if is_method:
@@ -502,12 +587,12 @@ def function_call_code_tags(fn_name, args_pattern, is_method=False):
 
     # Make code tag and detailed code tag:
     code_tag = "<code>{}</code>".format(
-        html.escape(specific_names[0])
+        escape(specific_names[0])
     )
 
     details_code = phrasing.comma_list(
         [
-            "<code>{}</code>".format(html.escape(name))
+            "<code>{}</code>".format(escape(name))
             for name in specific_names
         ],
         junction="or"
