@@ -26,10 +26,11 @@ def simplify(description):
     Normalizes case and removes HTML tags from the given goal
     description, for use in expectation matching. Note that angle
     brackets which aren't used for HTML tags are assumed to already be
-    escaped.
+    escaped. Adds '^^^' at the start and '$$$' at the end so that rules
+    can use those anchors (or part of them) for disambiguation.
     """
     stripped = re.sub(r"<[^>]*>", '', description)
-    return stripped.casefold()
+    return '^^^' + stripped.casefold() + '$$$'
 
 
 def categorized_goal_path(goal_type, category, *descriptions):
@@ -58,27 +59,31 @@ class Expectation:
     primary obfuscated description entry for each goal/category. The
     specified string only has to match part of the goal description, but
     it must not match multiple goal descriptions at a given table level.
+    The characters '^^^' are added to the beginning of the rubric string,
+    and '$$$' to the end, to aid in disambiguation.
 
     Because of these matching rules, for a rubric where the standard
     metric `potluck.rubrics.core_extras_categorized_metric` is used,
     goal paths are usually straightforward to construct when default
     descriptions are in place. Some examples:
 
-        For a core FunctionDef Check for function 'foo':
-            [ "procedure", "core", "define foo" ]
+    - For a core FunctionDef Check for function 'foo':
+       `[ "procedure", "core", "define foo" ]`
 
-        For an extra FunctionCall Check for 'bar' as a sub-rule of the
-        check above:
-            [ "procedure", "extra", "define foo", "call bar" ]
+    - For an extra FunctionCall Check for 'bar' as a sub-rule of the
+      check above:
+       `[ "procedure", "extra", "define foo", "call bar" ]`
 
-        For a core trace test of function 'foo':
-            [ "process", "core", "the foo function must" ]
+    - For a core trace test of function 'foo':
+       `[ "process", "core", "the foo function must" ]`
+    - (note that one could also use:)
+       `[ "process", "core", "^the foo function must" ]`
 
-        For a core result value test of function 'foo':
-            [ "product", "core", "foo returns" ]
+    - For a core result value test of function 'foo':
+       `[ "product", "core", "foo returns" ]`
 
-        For a core printed output test of function 'foo':
-            [ "behavior", "core", "foo prints" ]
+    - For a core printed output test of function 'foo':
+       `[ "behavior", "core", "foo prints" ]`
     """
     def __init__(self, goal_path, expected_status):
         """
@@ -93,6 +98,17 @@ class Expectation:
         `potluck.rubrics.Rubric` object formulates its overall report,
         because any top-level organizational report rows (e.g. for goal
         types or categories) need to be accounted for.
+
+        For matching goals, the case-folded version of each goal_path
+        entry is checked using 'in' against a case-folded version of each
+        rubric entry at the relevant level. Exactly 1 rubric entry must
+        match.  The rubric entries also have HTML tags stripped out, and
+        have '^^^' added at the front and '$$$' at the end to aid in
+        disambiguation.
+
+        For example, if there are rubric entries named "Bug #1" and
+        "Bug #11", an expectation for the "Bug #1" rubric entry could use
+        "bug #1$" as its goal_path entry.
         """
         self.goal_path = goal_path
         self.expected_status = expected_status
@@ -290,10 +306,16 @@ def check_entire_report(
 def example(username):
     """
     Registers a current username such that calls to expect create
-    expectations for that example submission.
+    expectations for that example submission, and creates an entry in the
+    expectations table for it so that even if no expectations are
+    established it will still be tested using default expectations.
     """
     global CURRENT_EXAMPLE
     CURRENT_EXAMPLE = username
+    mname = file_utils.get_spec_module_name()
+    EXPECTATIONS\
+        .setdefault(mname, {})\
+        .setdefault(username, [])
 
 
 def expect(status, goal_type, category, *path):
@@ -301,14 +323,14 @@ def expect(status, goal_type, category, *path):
     Creates an `Expectation` object and registers it under a given task
     ID and username. Arguments are:
 
-        status: The expected status. See `potluck.rubrics.Goal`.
-        taskid: The task ID this expectation applies to.
-        username: The username whose submission the expectation applies
-            to (usually a fake user set up for testing purposes).
-        goal_type: The type of goal.
-        category: The goal category.
-        path: One or more additional strings specifying which goal we're
-            targeting (see `Expectation`).
+    - status: The expected status. See `potluck.rubrics.Goal`.
+    - taskid: The task ID this expectation applies to.
+    - username: The username whose submission the expectation applies
+        to (usually a fake user set up for testing purposes).
+    - goal_type: The type of goal.
+    - category: The goal category.
+    - path: One or more additional strings specifying which goal we're
+        targeting (see `Expectation`).
     """
     mname = file_utils.get_spec_module_name()
     EXPECTATIONS\
