@@ -1,3 +1,9 @@
+/* jshint esversion: 6 */
+
+/*--------------------*
+ * Code line focusing *
+ *--------------------*/
+
 var CODEFOCUS = null;
 
 function transition(id, classA, classB) {
@@ -37,9 +43,13 @@ function unfocusCodeLine() {
     CODEFOCUS = null;
 }
 
-function focusCodeLine(codeid, lineid) {
+function focusCodeLine(lineid) {
     unfocusCodeLine();
-    transition(codeid, "hide", "show");
+    let line = document.getElementById(lineid);
+    let bin = sticky_bin_ancestor(line);
+    if (bin) {
+        ensure_bin_is_visible(bin);
+    }
     transition(lineid, "codeline_plain", "codeline_focus");
     CODEFOCUS = lineid;
 }
@@ -79,7 +89,7 @@ function openCodeLine(codeid, lineid, toggle_id, on_symbol) {
         }
         transition(lineid + "_refs", "hide", "show");
     }
-    focusCodeLine(codeid, lineid);
+    focusCodeLine(lineid);
 }
 
 function toggleSection(id, toggle_id, on_symbol, off_symbol) {
@@ -87,8 +97,9 @@ function toggleSection(id, toggle_id, on_symbol, off_symbol) {
 }
 
 function toggleRow(id, toggle_id, on_symbol, off_symbol) {
-    // Either might be missing, but if either toggles, so should the symbol
-    // (it won't toggle twice since it detects the show/hide destination state)
+    // Either might be missing, but if either toggles, so should the
+    // symbol (it won't toggle twice since it detects the show/hide
+    // destination state)
     toggle("row_" + id, "show", "hide", toggle_id, on_symbol, off_symbol);
     toggle("ul_" + id, "show", "hide", toggle_id, on_symbol, off_symbol);
 }
@@ -121,12 +132,44 @@ function toggleClass(classs, classA, classB) {
     return null;
 }
 
-function togglePoints() {
-    toggleClass("autoreport_points", "points_hide", "show");
-    toggleClass("regrade", "regrade_hide", "regrade_show");
+
+/*
+ * Event handler for code lines which focuses them.
+ */
+function focusThisCodeLine() {
+    focusCodeLine(event.currentTarget.id);
+    return false; // prevent further event processing
 }
 
 
+/*
+ * Event handler for linerefs which focuses the destination line
+ */
+function focusDestinationLine() {
+    let me = event.currentTarget;
+    let line_id = me.getAttribute("data-lineid");
+    focusCodeLine(line_id);
+    return true; // allow link to also trigger anchor mechanism
+}
+
+
+// Add click handlers to each code line and each lineref link
+window.addEventListener("load", function () {
+    let codelines = document.querySelectorAll("span.codeline");
+    for (let line of codelines) {
+        line.addEventListener("click", focusThisCodeLine);
+    }
+
+    let linerefs = document.querySelectorAll("a.lineref");
+    for (let ref of linerefs) {
+        ref.addEventListener("click", focusDestinationLine);
+    }
+});
+
+/*------*
+ * Tabs *
+ *------*/
+
 function toggleTab() {
     // Click event handler for anchors in a tabs setup to hide all other
     // tabs except the one that corresponds to the one we clicked on.
@@ -169,12 +212,23 @@ function toggleTab() {
 }
 
 
+
+// Add click handlers to every tab
+window.addEventListener("load", function () {
+    let tab_buttons = document.querySelectorAll("div.tabs li.tab");
+    for (let button of tab_buttons) {
+        button.addEventListener("click", toggleTab);
+    }
+});
+
 /*----------------------*
  * Feedback interaction *
  *----------------------*/
 
 window.addEventListener("load", function () {
-    let hide_accomplished = document.getElementById("hide_accomplished_goals")
+    let hide_accomplished = document.getElementById(
+        "hide_accomplished_goals"
+    );
     if (hide_accomplished != null) {
         hide_accomplished.addEventListener(
             "change",
@@ -302,7 +356,7 @@ function stop_floating(bin) {
     bin.style.removeProperty("width");
     bin.style.removeProperty("height");
 
-    let label = bin.querySelector(".bin_label")
+    let label = bin.querySelector(".bin_label");
     label.style.removeProperty("position");
     label.style.removeProperty("top");
     label.style.removeProperty("left");
@@ -455,11 +509,20 @@ function toggle_bin_state(bin, maximize) {
 
 
 // Ensures that a specific sticky bin is currently visible, either by
-// maximizing it if it's floating, or doing nothing if it's not floating.
+// maximizing it if it's floating and not already maximized, or by doing
+// nothing if it's not floating, or if it's floating and already
+// maximized.
 function ensure_bin_is_visible(bin) {
-    if (bin.classList.contains("sticky_floating") && !bin.classList.contains("sticky_maximized")) {
-
+    // if we're already floating
+    if (bin.classList.contains("sticky_floating")) {
+        // ...but we're not maximized
+        if (!bin.classList.contains("sticky_maximized")) {
+            // maximize us
+            toggle_bin_state(bin, true); // maximize it
+        }
+        // else we're already visible (floating & maximized)
     }
+    // else we're already visible (not floating)
 }
 
 // Returns an ancestor element of the given element that is a sticky bin,
@@ -523,10 +586,12 @@ window.addEventListener("load", function () {
             if (bin != null) {
                 // If the destination's in a sticky bin, add a click handler
                 // to the link to open that bin
+                /* jshint -W083 */
                 link.addEventListener("click", function () {
                     toggle_bin_state(bin, true);
                     return true; // continue with normal event operation
                 });
+                /* jshint +W083 */
             }
         }
     }
