@@ -160,9 +160,9 @@ base.refine(
     .compare_strings_firmly()\
     .set_goal_description(
         (
-            "bug #6",
+            "bug #11", # numbered 11 to test meta goal-name-disambiguation
             "Hint: Can you add a number to a string?",
-            "bug #6 (concatenation vs. addition)",
+            "bug #11 (concatenation vs. addition)",
             (
                 "Online 34, concatenation of strings is used to build"
                 " a prompt. However, while <code>firstNum</code> is"
@@ -205,17 +205,17 @@ from potluck import meta # noqa E402
 
 meta.example("imperfect")
 
-for bug_nr in [1, 2, 3, 5, 6]:
+for bug_nr in [1, 2, 3, 5, 11]:
     meta.expect(
         "failed",
         "behavior",
         "core",
-        f"bug #{bug_nr}"
+        f"bug #{bug_nr}$"
     )
 
 meta.expect(
     "partial",
     "behavior",
     "core",
-    "bug #4"
+    "bug #4$"
 )
