@@ -141,6 +141,16 @@ spec.DontWasteFruit()
 spec.DontWasteBoxes()
 spec.RequireDocstrings()
 
+# Testing goals
+
+spec.RequireExpectations({
+    "indentMessage": 2,
+    "printMessage": 2,
+    "ellipseArea": 2
+})
+spec.ObserveExpectationResults()
+spec.ValidateExpectations(["indentMessage", "printMessage", "ellipseArea"])
+
 # Construct our rubric
 rubric = spec.rubric()
 
@@ -152,7 +162,7 @@ meta.example("imperfect")
 
 meta.expect("partial", "style", "core", "documented")
 meta.expect("partial", "style", "extra", "ignore the results")
-meta.expect("failed", "procedure", "core", "define printMessage")
+meta.expect("partial", "procedure", "core", "define printMessage")
 meta.expect("accomplished", "procedure", "core", "define printMessage",
             "call print")
 meta.expect("failed", "procedure", "core", "define printMessage",
@@ -167,3 +177,6 @@ meta.expect("failed", "process", "extra", "maintain invariants")
 meta.expect("failed", "product", "core", "ellipseArea")
 meta.expect("failed", "behavior", "core", "correct indentation")
 meta.expect("failed", "behavior", "extra", "correct output")
+meta.expect("partial", "testing", "core", "defines tests")
+meta.expect("partial", "testing", "extra", "were met")
+meta.expect("partial", "testing", "extra", "are correct")
