@@ -4,6 +4,8 @@ Runs tests via pytest. Invoke using `python -m potluck.tests`.
 tests/__main__.py
 """
 
+import sys
+
 import pytest
 
-pytest.main(["--pyargs", "potluck.tests"])
+sys.exit(pytest.main(["--pyargs", "potluck.tests"]))
