@@ -37,13 +37,14 @@ RUBRIC_EXPECTS = {
         ),
         "<code>ellipseArea</code> returns the correct result",
     ],
+    "freedomTest": [ "<title>freedomTest Rubric</title>" ],
 }
 
 # Expectations about reports
 REPORT_EXPECTS = {
     "functionsTest": {
         "perfect": { "evaluation": "excellent" },
-        "imperfect": { "evaluation": "incomplete" },
+        "imperfect": { "evaluation": "partially complete" },
     },
     "debugTest": {
         "perfect": { "evaluation": "excellent" },
@@ -56,12 +57,22 @@ REPORT_EXPECTS = {
     "interactiveTest": {
         "perfect": { "evaluation": "excellent" },
         "imperfect": { "evaluation": "partially complete" },
+    },
+    "freedomTest": {
+        "perfect": { "evaluation": "excellent" },
+        "imperfect": { "evaluation": "partially complete" },
     }
 }
 
 
 @pytest.fixture(
-    params=["functionsTest", "debugTest", "interactiveTest", "sceneTest"]
+    params=[
+        "functionsTest",
+        "debugTest",
+        "interactiveTest",
+        "sceneTest",
+        "freedomTest"
+    ]
 )
 def taskid(request):
     """
@@ -172,6 +183,9 @@ def test_rubric_creation(in_evaldir, taskid, logfile, rubricfile):
 
 
 def test_evaluation(in_evaldir, taskid, username, reportfiles, logfile):
+    """
+    Tests the potluck_eval script for a certain task/user example.
+    """
     assert not logfile.exists()
     r_json, r_html = reportfiles
     assert not r_json.exists()
